package writecache

import (
	"fmt"
	"io"

	common "git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal"
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/kvio"
	oid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object/id"
	"github.com/spf13/cobra"
)

var listCMD = &cobra.Command{
	Use:   "inspect",
	Short: "Object inspection",
	Long:  `Inspect specific object in a write-cache.`,
	Run:   listFunc,
}

func init() {
	common.AddComponentPathFlag(listCMD, &vPath)
}

func listFunc(cmd *cobra.Command, _ []string) {
	// other targets can be supported
	w := cmd.OutOrStderr()

	db := openWC(cmd)
	defer db.Close()

	err := kvio.ForEachKey(db, func(k kvio.Key) error {
		var addr oid.Address
		if err := addr.DecodeString(string(k)); err != nil {
			return fmt.Errorf("could not parse object address: %w", err)
		}
		_, err := io.WriteString(w, fmt.Sprintf("%s\n", addr))
		return err
	})

	common.ExitOnErr(cmd, common.Errf("write-cache iterator failure: %w", err))
}
