package badger

import (
	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/local_object_storage/kvio"
	badger "github.com/dgraph-io/badger/v4"
)

type txImpl struct {
	tx *badger.Txn
}

func (tx txImpl) Read(k kvio.Key, f func(kvio.Value) error) error {
	it, err := tx.tx.Get(k)
	if err != nil {
		if err == badger.ErrKeyNotFound {
			return f(nil)
		}
		return err
	}
	return it.Value(func(val []byte) error { return f(val) })
}

func (tx txImpl) Write(k kvio.Key, v kvio.Value) error { return tx.tx.Set(k, v) }
func (tx txImpl) Delete(k kvio.Key) error              { return tx.tx.Delete(k) }

func (tx txImpl) IterateKeys() kvio.KeyCursor {
	opts := badger.DefaultIteratorOptions
	opts.PrefetchValues = false
	cur := tx.tx.NewIterator(opts)
	cur.Rewind()
	return cursor{cur}
}

func (tx txImpl) Iterate() kvio.Cursor {
	cur := tx.tx.NewIterator(badger.DefaultIteratorOptions)
	cur.Rewind()
	return cursor{cur}
}

type writeBatchTx struct {
	wb *badger.WriteBatch
}

func (tx writeBatchTx) Write(k kvio.Key, v kvio.Value) error { return tx.wb.Set(k, v) }
func (tx writeBatchTx) Delete(k kvio.Key) error              { return tx.wb.Delete(k) }
