package metrics

import (
	"strconv"
	"time"

	"git.frostfs.info/TrueCloudLab/frostfs-observability/metrics"
	"github.com/prometheus/client_golang/prometheus"
)

const treeServiceLabelSuccess = "success"

type TreeMetricsRegister interface {
	AddReplicateTaskDuration(time.Duration, bool)
	AddReplicateWaitDuration(time.Duration, bool)
	AddSyncDuration(time.Duration, bool)
}

type treeServiceMetrics struct {
	replicateTaskDuration *prometheus.HistogramVec
	replicateWaitDuration *prometheus.HistogramVec
	syncOpDuration        *prometheus.HistogramVec
}

var _ TreeMetricsRegister = (*treeServiceMetrics)(nil)

func newTreeServiceMetrics() *treeServiceMetrics {
	const treeServiceSubsystem = "treeservice"
	return &treeServiceMetrics{
		replicateTaskDuration: metrics.NewHistogramVec(prometheus.HistogramOpts{
			Namespace: namespace,
			Subsystem: treeServiceSubsystem,
			Name:      "replicate_task_duration_seconds",
			Help:      "Duration of individual replication tasks executed as part of replication loops",
		}, []string{treeServiceLabelSuccess}),
		replicateWaitDuration: metrics.NewHistogramVec(prometheus.HistogramOpts{
			Namespace: namespace,
			Subsystem: treeServiceSubsystem,
			Name:      "replicate_wait_duration_seconds",
			Help:      "Duration of overall waiting time for replication loops",
		}, []string{treeServiceLabelSuccess}),
		syncOpDuration: metrics.NewHistogramVec(prometheus.HistogramOpts{
			Namespace: namespace,
			Subsystem: treeServiceSubsystem,
			Name:      "sync_duration_seconds",
			Help:      "Duration of synchronization operations",
		}, []string{treeServiceLabelSuccess}),
	}
}

func (m *treeServiceMetrics) AddReplicateTaskDuration(d time.Duration, success bool) {
	m.replicateTaskDuration.With(prometheus.Labels{
		treeServiceLabelSuccess: strconv.FormatBool(success),
	}).Observe(d.Seconds())
}

func (m *treeServiceMetrics) AddReplicateWaitDuration(d time.Duration, success bool) {
	m.replicateWaitDuration.With(prometheus.Labels{
		treeServiceLabelSuccess: strconv.FormatBool(success),
	}).Observe(d.Seconds())
}

func (m *treeServiceMetrics) AddSyncDuration(d time.Duration, success bool) {
	m.syncOpDuration.With(prometheus.Labels{
		treeServiceLabelSuccess: strconv.FormatBool(success),
	}).Observe(d.Seconds())
}
